package cz.cvut.k36.pr2.hw.hw02;

/**
 * The RGB color representation.
 */
public class Color {

    private final double red,
                         green,
                         blue;

    /**
     * Constructs the Color with given color components. Each component must be
     * in rage [0, 255]. The 0 represents minimal value and 255 represents maximal value.
     * If any of the given components is not within this range the
     * {@link java.lang.IllegalArgumentException} is thrown
     * @param red component
     * @param green compoment
     * @param blue component
     * @throws java.lang.IllegalArgumentException if any of the given components is not withnin range
     */
    public Color(double red, double green, double blue) {
        this.red = validate(red, "red");
        this.green = validate(green, "green");
        this.blue = validate(blue, "blue");
    }

    private double validate(double value, String valueName) {
        if (value < 0 || value > 255) {
            throw new IllegalArgumentException("Color " + valueName + " is not withing bounds <0; 255>: " + value);
        }
        return value;
    }

    public double getRed() {
        return red;
    }

    public double getGreen() {
        return green;
    }

    public double getBlue() {
        return blue;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Color)) return false;

        Color color = (Color) o;

        if (Double.compare(color.blue, blue) != 0) return false;
        if (Double.compare(color.green, green) != 0) return false;
        if (Double.compare(color.red, red) != 0) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result;
        long temp;
        temp = Double.doubleToLongBits(red);
        result = (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(green);
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(blue);
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        return result;
    }

    @Override
    public String toString() {
        return "Color(" +
                "R:" + red +
                ", G: " + green +
                ", B: " + blue +
                ')';
    }
}
